from .backends import RedisCache, SimpleMemoryCache
from .utils import cached


__all__ = (
    'RedisCache',
    'SimpleMemoryCache',
    'cached',
)


default_cache = None


def config_default_cache(*args, **kwargs):
    global default_cache
    backend = kwargs.pop('backend', SimpleMemoryCache)
    default_cache = backend(*args, **kwargs)
    return default_cache



import logging

from collections import deque


logger = logging.getLogger(__name__)


class DefaultPolicy:
    """
    Default and base policy. It's the default used by all backends and it does nothing.

    :param client: Backend class to interact with the storage.
    """

    def __init__(self, client):
        self.client = client

    async def pre_get(self, key):
        pass

    async def post_get(self, key):
        pass

    async def pre_set(self, key, value):
        pass

    async def post_set(self, key, value):
        pass


class LRUPolicy(DefaultPolicy):
    """
    Implements a Least Recently Used policy with max_keys. The policy does the following:

        - When a key is retrieved (get or mget), keys are moved to the beginning of the queue
        - When a key is added (set, mset or add), keys are added to the beginning of the queue. If
            the queue is full, it will remove as many keys as needed to make space for the new
            ones.
    """
    def __init__(self, *args, max_keys=None, **kwargs):
        super().__init__(*args, **kwargs)
        if max_keys is not None:
            assert max_keys >= 1, "Number of keys must be 1 or bigger"
        self.dq = deque(maxlen=max_keys)

    async def post_get(self, key):
        """
        Remove the key from its current position and set it at the beginning of the queue.

        :param key: string key used in the get operation
        """
        self.dq.remove(key)
        self.dq.appendleft(key)

    async def post_set(self, key, value):
        """
        Set the given key at the beginning of the queue. If the queue is full, remove the last
        item first.

        :param key: string key used in the set operation
        :param value: obj used in the set operation
        """
        if len(self.dq) == self.dq.maxlen:
            await self.client.delete(self.dq.pop())
        self.dq.appendleft(key)



import abc
import logging


logger = logging.getLogger(__name__)

try:
    import ujson as json
except ImportError:
    logger.warning("ujson module not found, usin json")
    import json

try:
    import cPickle as pickle
except ImportError:
    logger.warning("cPickle module not found, using pickle")
    import pickle


class BaseSerializer(metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def dumps(self, value):
        pass

    @abc.abstractmethod
    def loads(self, value):
        pass


class DefaultSerializer(BaseSerializer):
    """
    Dummy serializer that returns the same value passed both in serialize and
    deserialize methods.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def dumps(self, value):
        return value

    def loads(self, value):
        return value


class PickleSerializer(BaseSerializer):
    """
    Transform data to bytes using pickle.dumps and pickle.loads to retrieve it back.
    """

    encoding = None

    def dumps(self, value):
        """
        Serialize the received value using ``pickle.dumps``.

        :param value: obj
        :returns: bytes
        """
        return pickle.dumps(value)

    def loads(self, value):
        """
        Deserialize value using ``pickle.loads``.

        :param value: bytes
        :returns: obj
        """
        if value is None:
            return None
        return pickle.loads(value)


class JsonSerializer(BaseSerializer):
    """
    Transform data to json string with json.dumps and json.loads to retrieve it back.
    """

    def dumps(self, value):
        """
        Serialize the received value using ``json.dumps``.

        :param value: dict
        :returns: str
        """
        return json.dumps(value)

    def loads(self, value):
        """
        Deserialize value using ``json.loads``.

        :param value: str
        :returns: dict
        """
        return json.loads(value)



import logging
import aiocache

from aiocache import SimpleMemoryCache
from aiocache.serializers import DefaultSerializer


logger = logging.getLogger(__name__)


def cached(*args, ttl=0, backend=None, serializer=None, **kwargs):
    """
    Caches the functions return value into a key generated with module_name, function_name and args.

    In some cases you will need to send more args than just the ttl, backend and serializer.
    An example would be endpoint and port for the RedisBackend. This extra args will be propagated
    to the backend class when instantiating.

    :param ttl: int seconds to store the function call. Default is 0
    :param backend: backend class to use when calling the ``set``/``get`` operations. Default is
        :class:`aiocache.backends.SimpleMemoryCache`
    :param serializer: serializer instance to use when calling the ``serialize``/``deserialize``.
        Default is :class:`aiocache.serializers.DefaultSerializer`
    """
    cache = get_default_cache(backend=backend, serializer=serializer, *args, **kwargs)

    def cached_decorator(fn):
        async def wrapper(*args, **kwargs):
            key = (fn.__module__ or "stub") + fn.__name__ + str(args) + str(kwargs)
            if await cache.exists(key):
                return await cache.get(key)
            else:
                res = await fn(*args, **kwargs)
                await cache.set(key, res, ttl=ttl)
                return res
        return wrapper
    return cached_decorator


def get_default_cache(*args, backend=None, serializer=None, **kwargs):
    serializer = serializer if serializer else DefaultSerializer()
    if backend:
        return backend(serializer=serializer, *args, **kwargs)
    elif aiocache.default_cache:
        return aiocache.default_cache
    else:
        return SimpleMemoryCache(serializer=serializer, *args, **kwargs)



from setuptools import setup

install_requires = []
tests_require = install_requires + ['pytest']

setup(
    name="aiocache",
    version="0.0.4",
    author="Manuel Miranda",
    url="https://github.com/argaen/aiocache",
    author_email="manu.mirandad@gmail.com",
    description="Asynchronous redis cache",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
    ],
    packages=['aiocache'],
    install_requires=install_requires,
    setup_requires=['pytest-runner'],
    tests_require=tests_require,
)
